@@ -19,12 +19,14 @@ use goat_cli::utils::{
 #[tokio::main]
 async fn main() -> Result<()> {
     let matches = Command::new("goat")
+        // to fix the binary name in the help messages
+        .bin_name("goat")
         .version(crate_version!())
         .propagate_version(true)
         .arg_required_else_help(true)
         .global_setting(AppSettings::DeriveDisplayOrder)
         .author("Max Brown, Richard Challis, Sujai Kumar, Cibele Sotero-Caio <goat@genomehubs.org>")
-        .about("Genomes on a Tree. Query metadata across the tree of life.")
+        .about("Genomes on a Tree. Query metadata across the tree of life.\n\nFor a tutorial on usage, visit: https://github.com/genomehubs/goat-cli/wiki\nVisit the GoaT website here: https://goat.genomehubs.org/")
         .subcommand(
             Command::new("search")
                 .about("Query metadata for any taxon across the tree of life.")
@@ -232,6 +234,11 @@ async fn main() -> Result<()> {
                         .long("print-expression")
                         .help("Print all variables in GoaT currently, with their associated variants.\nUseful for construction of expressions.")
                 )
+                .arg(
+                    Arg::new("progress-bar")
+                        .long("progress-bar")
+                        .help("Add a progress bar to large queries, to estimate time left.")
+                )
                 .arg(
                     Arg::new("url")
                         .short('u')
@@ -447,6 +454,11 @@ async fn main() -> Result<()> {
                         .long("print-expression")
                         .help("Print all variables in GoaT currently, with their associated variants.\nUseful for construction of expressions.")
                 )
+                .arg(
+                    Arg::new("progress-bar")
+                        .long("progress-bar")
+                        .help("Add a progress bar to large queries, to estimate time left.")
+                )
                 .arg(
                     Arg::new("url")
                         .short('u')
@@ -465,6 +477,14 @@ async fn main() -> Result<()> {
                         .required_unless_present("file")
                         .help("The taxon to search. An NCBI taxon ID, or the name of a taxon at any rank."),
                 )
+                .arg(
+                    Arg::new("file")
+                        .short('f')
+                        .long("file")
+                        .takes_value(true)
+                        .required_unless_present_any(["taxon"])
+                        .help(&format!("A file of NCBI taxonomy ID's (tips) and/or binomial names.\nEach line should contain a single entry.\nFile size is limited to {} entries.", pretty_print_usize(*UPPER_CLI_FILE_LIMIT))[..]),
+                )
                 .arg(
                     Arg::new("url")
                         .short('u')
@@ -504,15 +524,27 @@ async fn main() -> Result<()> {
                         .possible_values(&["species", "genus", "family", "order"])
                         .help("The number of results to return."),
                 )
+                .arg(
+                    Arg::new("progress-bar")
+                        .long("progress-bar")
+                        .help("Add a progress bar to large queries, to estimate time left.")
+                ),
         )
         .get_matches();
 
     match matches.subcommand() {
         Some(("search", matches)) => {
-            try_join!(
-                run::search(&matches),
-                progress::progress_bar(&matches, "search")
-            )?;
+            let progress_bar = matches.is_present("progress-bar");
+
+            match progress_bar {
+                true => {
+                    try_join!(
+                        run::search(&matches),
+                        progress::progress_bar(&matches, "search")
+                    )?;
+                }
+                false => run::search(&matches).await?,
+            }
         }
         Some(("count", matches)) => {
             count::count(&matches, true, false).await?;
@@ -521,10 +553,17 @@ async fn main() -> Result<()> {
             lookup::lookup(&matches, true).await?;
         }
         Some(("newick", matches)) => {
-            try_join!(
-                newick::get_newick(matches),
-                progress::progress_bar(&matches, "newick")
-            )?;
+            let progress_bar = matches.is_present("progress-bar");
+
+            match progress_bar {
+                true => {
+                    try_join!(
+                        newick::get_newick(matches),
+                        progress::progress_bar(&matches, "newick")
+                    )?;
+                }
+                false => newick::get_newick(matches).await?,
+            }
         }
         _ => {
             unreachable!()
