@@ -1,11 +1,10 @@
-// parse expressions on the command line
-// example expressions will be:
-// plastid_gc_percent < 33 AND gc_percent > 55
-// TODO: ADD TAX_RANKS?
+// TODO:
+// - type checks on expression (https://github.com/genomehubs/goat-cli/issues/3)
+// -
 
 use crate::error::error::ExpressionParseError;
 use crate::utils::tax_ranks::TaxRanks;
-use crate::utils::utils::switch_string_to_url_encoding;
+use crate::utils::utils::{did_you_mean, switch_string_to_url_encoding};
 use crate::utils::variable_data::GOAT_VARIABLE_DATA;
 
 use anyhow::{bail, ensure, Result};
@@ -13,18 +12,76 @@ use regex::{CaptureMatches, Captures, Regex};
 use std::fmt;
 use tabled::{Footer, Header, MaxWidth, Modify, Rows, Table, Tabled};
 
+/// Serialize GoaT variables into their types.
+///
+/// See [here](https://www.elastic.co/guide/en/elasticsearch/reference/current/number.html)
+/// for more details.
 #[derive(Tabled)]
 pub enum TypeOf<'a> {
+    /// Signed 64 bit int.
     Long,
+    /// Signed 16 bit int.
     Short,
+    /// Float with one decimal place.
     OneDP,
+    /// Float with two decimal places.
     TwoDP,
+    /// Signed 32 bit int.
     Integer,
+    /// A date.
     Date,
+    /// Half precision 16 bit float.
     HalfFloat,
+    /// A variable which itself is an enumeration.
     Keyword(Vec<&'a str>),
 }
 
+impl<'a> TypeOf<'a> {
+    /// Check the values input by a user, so `goat-cli` displays meaningful help.
+    fn check(&self, other: &str, variable: &str) -> Result<()> {
+        // we will have to parse the `other` conditionally on what the
+        // `TypeOf` is.
+        let _ = match self {
+            TypeOf::Long => match other.parse::<i64>() {
+                Ok(_) => (),
+                Err(_) => bail!(format!("For variable \"{variable}\" in the expression, an input error was found. Pass an integer as a value.")),
+            },
+            TypeOf::Short => match other.parse::<i16>() {
+                Ok(_) => (),
+                Err(_) => bail!(format!("For variable \"{variable}\" in the expression, an input error was found. Pass an integer as a value.")),
+            },
+            TypeOf::OneDP => match other.parse::<f32>() {
+                Ok(_) => (),
+                Err(_) => bail!(format!("For variable \"{variable}\" in the expression, an input error was found. Pass a float as a value.")),
+            },
+            TypeOf::TwoDP => match other.parse::<f32>() {
+                Ok(_) => (),
+                Err(_) => bail!(format!("For variable \"{variable}\" in the expression, an input error was found. Pass a float as a value.")),
+            },
+            TypeOf::Integer => match other.parse::<i32>() {
+                Ok(_) => (),
+                Err(_) => bail!(format!("For variable \"{variable}\" in the expression, an input error was found. Pass an integer as a value.")),
+            },
+            // dates should be in a specified format
+            // yyyy-mm-dd
+            TypeOf::Date => {
+                let tokens = other.split("-").collect::<Vec<_>>();
+                ensure!(
+                    tokens.len() == 1 || tokens.len() == 3,
+                    "Improperly formatted date. Please make sure date is in the format yyyy-mm-dd, or yyyy."
+                )
+            }
+            TypeOf::HalfFloat => match other.parse::<f32>() {
+                Ok(_) => (),
+                Err(_) => bail!(format!("For variable \"{variable}\" in the expression, an input error was found. Pass a float as a value.")),
+            },
+            // keywords handled elsewhere
+            TypeOf::Keyword(_) => (),
+        };
+        Ok(())
+    }
+}
+
 impl<'a> fmt::Display for TypeOf<'a> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match &*self {
@@ -43,7 +100,9 @@ impl<'a> fmt::Display for TypeOf<'a> {
     }
 }
 
-// kind of an option alias here.
+/// Kind of an option alias. Does a
+/// particular variable have a function
+/// associated with it? Usually min/max.
 pub enum Function<'a> {
     None,
     Some(Vec<&'a str>),
@@ -58,6 +117,7 @@ impl<'a> fmt::Display for Function<'a> {
     }
 }
 
+/// The GoaT variable of interest.
 #[derive(Tabled)]
 pub struct Variable<'a> {
     #[tabled(rename = "Display Name")]
@@ -68,9 +128,11 @@ pub struct Variable<'a> {
     pub functions: Function<'a>,
 }
 
+/// The column headers for `goat-cli search --print-expression`.
 #[derive(Tabled)]
 struct ColHeader(#[tabled(rename = "Expression Name")] &'static str);
 
+/// Print the table of GoaT variable data.
 pub fn print_variable_data() {
     // for some space
     println!("");
@@ -103,6 +165,7 @@ pub fn print_variable_data() {
     println!("{}", table_string);
 }
 
+/// The CLI expression which needs to be parsed.
 pub struct CLIexpression<'a> {
     pub inner: &'a str,
     pub length: usize, // these queries can't be crazy long.
@@ -110,6 +173,7 @@ pub struct CLIexpression<'a> {
 }
 
 impl<'a> CLIexpression<'a> {
+    /// Constructor for [`CLIexpression`].
     pub fn new(string: &'a str) -> Self {
         Self {
             inner: string,
@@ -118,6 +182,7 @@ impl<'a> CLIexpression<'a> {
         }
     }
 
+    /// The initial split on the keyword `AND`.
     fn split(&self) -> Self {
         let mut res_vec = Vec::new();
         // commands only accept AND? Rich!
@@ -137,6 +202,8 @@ impl<'a> CLIexpression<'a> {
         }
     }
 
+    /// The main function which parses a [`CLIexpression`]. A bit of a
+    /// monster of a function. Might need cleaning up at some point.
     pub fn parse(&mut self) -> Result<String> {
         if self.length > 100 {
             bail!(ExpressionParseError::QueryTooLong)
@@ -224,23 +291,48 @@ impl<'a> CLIexpression<'a> {
             // check this vector is length 3 or 1
             ensure!(
                     curr_el_vec.len() == 3 || curr_el_vec.len() == 1,
-                    "[-]\tSplit vector on single expression is invalid - length = {}. Are the input variables or operands correct?",
+                    "Split vector on single expression is invalid - length = {}. Are the input variables or operands correct?",
                     curr_el_vec.len()
                 );
             match curr_el_vec.len() {
                 3 => {
                     // trim strings
-                    let variable = curr_el_vec[0].trim();
+                    // replace rogue quotes (not sure why this is happening now, but was not before...)
+                    // manually escape these...
+                    let variable = &curr_el_vec[0].trim().replace("\"", "").replace("'", "")[..];
                     let operator = switch_string_to_url_encoding(curr_el_vec[1])?.trim();
-                    // let operator = curr_el_vec[1];
-                    let value = curr_el_vec[2].trim();
+                    let value = &curr_el_vec[2].trim().replace("\"", "").replace("'", "")[..];
 
                     if !var_vec_check.contains(&variable)
                         && !var_vec_min_max_check.contains(&variable.to_string())
                     {
-                        // might be able to check max/min/length here.
-                        // e.g. max(gc_content) > 0.3
-                        bail!(ExpressionParseError::SplitVectorError)
+                        // ew
+                        // just combining the min/max and normal variable vectors
+                        // into a single vector.
+                        let combined_checks = var_vec_check
+                            .iter()
+                            .map(|e| String::from(*e))
+                            .collect::<Vec<String>>()
+                            .iter()
+                            .chain(
+                                var_vec_min_max_check
+                                    .iter()
+                                    .map(|e| String::from(e))
+                                    .collect::<Vec<String>>()
+                                    .iter(),
+                            )
+                            .map(|e| String::from(e))
+                            .collect::<Vec<String>>();
+
+                        let var_vec_mean = did_you_mean(&combined_checks, variable);
+
+                        if let Some(value) = var_vec_mean {
+                            bail!(
+                                "In your expression (LHS) you typed \"{}\" - did you mean \"{}\"?",
+                                variable,
+                                value
+                            )
+                        }
                     }
 
                     // this panics with min/max.
@@ -277,12 +369,19 @@ impl<'a> CLIexpression<'a> {
                                 })
                                 .collect::<Vec<String>>();
 
-                            if !value_split_commas.iter().all(|e| k.contains(&&e[..])) {
-                                // BUG: if there are parentheses in the enum variant
-                                // but I think GoaT central can fix this.
-                                // then the URL encoding fails.
-                                bail!(ExpressionParseError::KeywordEnumError)
+                            // now check our keyword enums
+                            for val in &value_split_commas {
+                                let possibilities =
+                                    k.iter().map(|e| String::from(*e)).collect::<Vec<_>>();
+                                let did_you_mean_str = did_you_mean(&possibilities, &val);
+
+                                if let Some(value) = did_you_mean_str {
+                                    if value != val.to_owned() {
+                                        bail!("In your expression (RHS) you typed \"{}\" - did you mean \"{}\"?", val, value)
+                                    }
+                                }
                             }
+
                             // now modify value_split_commas to parse parentheses
                             let parsed_value_split_commas = value
                                 .split(",")
@@ -308,7 +407,10 @@ impl<'a> CLIexpression<'a> {
                             // assume there is another expression to follow
                             expression += "AND%20"
                         }
-                        _ => {
+                        t => {
+                            // here can we type check input
+                            TypeOf::check(t, value, variable)?;
+
                             // build expression
                             expression += "%20";
                             expression += &url_encoded_variable;
@@ -341,9 +443,8 @@ impl<'a> CLIexpression<'a> {
     }
 }
 
-// thanks https://github.com/rust-lang/regex/issues/330
-// for splitting a string and keeping the delimiter.
-
+/// Split a string and keep the delimiter.
+/// Thanks [`BurntSushi`](https://github.com/rust-lang/regex/issues/330)
 #[derive(Debug)]
 struct SplitCaptures<'r, 't> {
     finder: CaptureMatches<'r, 't>,
@@ -356,7 +457,7 @@ impl<'r, 't> SplitCaptures<'r, 't> {
     pub fn new(re: &'r Regex, text: &'t str) -> SplitCaptures<'r, 't> {
         SplitCaptures {
             finder: re.captures_iter(text),
-            text: text,
+            text,
             last: 0,
             caps: None,
         }
